diff --git a/src/bin/pgbench/exprparse.y b/src/bin/pgbench/exprparse.y
index 25d5ad4..9cb47f0 100644
--- a/src/bin/pgbench/exprparse.y
+++ b/src/bin/pgbench/exprparse.y
@@ -194,6 +194,12 @@ static const struct
 	{
 		"random_zipfian", 3, PGBENCH_RANDOM_ZIPFIAN
 	},
+	{
+		"hash_fnv1a", 1, PGBENCH_HASH_FNV1A
+	},
+	{
+		"hash_murmur2", 1, PGBENCH_HASH_MURMUR2
+	},
 	/* keep as last array element */
 	{
 		NULL, 0, 0
diff --git a/src/bin/pgbench/pgbench.c b/src/bin/pgbench/pgbench.c
index 7ce6f60..0a32fac 100644
--- a/src/bin/pgbench/pgbench.c
+++ b/src/bin/pgbench/pgbench.c
@@ -61,6 +61,12 @@
 #define ERRCODE_UNDEFINED_TABLE  "42P01"
 
 /*
+ * Hashing constants
+ */
+#define FNV_PRIME 1099511628211
+#define FNV_OFFSET_BASIS 0xcbf29ce484222325
+
+/*
  * Multi-platform pthread implementations
  */
 
@@ -912,6 +918,50 @@ getZipfianRand(TState *thread, int64 min, int64 max, double s)
 }
 
 /*
+ * FNV-1a hash function
+ */
+static int64
+getHashFnv1a(int64 val)
+{
+	int64	result;
+	int		i;
+
+	result = FNV_OFFSET_BASIS;
+	for (i = 0; i < 8; ++i)
+	{
+		int32 octet = val & 0xff;
+
+		val = val >> 8;
+		result = result ^ octet;
+		result = result * FNV_PRIME;
+	}
+
+	return result;
+}
+
+static int64
+getHashMurmur2(int64 val)
+{
+	const uint64 m = 0xc6a4a7935bd1e995;
+	const int	r = 47;
+	uint64		result = val ^ (sizeof(int64) * m);
+	uint64		k = (uint64) val;
+
+	k *= m;
+	k ^= k >> r;
+	k *= m;
+
+	result ^= k;
+	result *= m;
+
+	result ^= result >> r;
+	result *= m;
+	result ^= result >> r;
+
+	return (int64) result;
+}
+
+/*
  * Initialize the given SimpleStats struct to all zeroes
  */
 static void
@@ -1850,6 +1900,22 @@ evalFunc(TState *thread, CState *st,
 				return true;
 			}
 
+			/* hashing */
+		case PGBENCH_HASH_FNV1A:
+		case PGBENCH_HASH_MURMUR2:
+			{
+				int64	val;
+				int64	result;
+
+				if (!coerceToInt(&vargs[0], &val))
+					return false;
+
+				result = (func == PGBENCH_HASH_FNV1A) ?
+					getHashFnv1a(val) : getHashMurmur2(val);
+				setIntValue(retval, result);
+				return true;
+			}
+
 		default:
 			/* cannot get here */
 			Assert(0);
diff --git a/src/bin/pgbench/pgbench.h b/src/bin/pgbench/pgbench.h
index 83fee1a..d00b195 100644
--- a/src/bin/pgbench/pgbench.h
+++ b/src/bin/pgbench/pgbench.h
@@ -76,7 +76,9 @@ typedef enum PgBenchFunction
 	PGBENCH_RANDOM,
 	PGBENCH_RANDOM_GAUSSIAN,
 	PGBENCH_RANDOM_EXPONENTIAL,
-	PGBENCH_RANDOM_ZIPFIAN
+	PGBENCH_RANDOM_ZIPFIAN,
+	PGBENCH_HASH_FNV1A,
+	PGBENCH_HASH_MURMUR2
 } PgBenchFunction;
 
 typedef struct PgBenchExpr PgBenchExpr;
